
#include <stdio.h>
#include "yacas.h"
#include "xxparser.h"
#include "standard.h"
#include "errors.h"
#include "unixcommandline.h"
#include "stdcommandline.h"

LispString the_out;
CYacas* yacas=NULL;
CCommandLine *commandline = NULL;

XXTokenizer xxtokenizer;
int iFinished = 0;
void LispXXTokenizer(LispEnvironment& aEnvironment, LispPtr& aResult,
                   LispPtr& aArguments)
{
    TESTARGS(1);
    aEnvironment.iCurrentTokenizer = &xxtokenizer;
    InternalTrue(aEnvironment,aResult);
}

void LispFinishPass(LispEnvironment& aEnvironment, LispPtr& aResult,
                   LispPtr& aArguments)
{
    iFinished = 1;
    InternalTrue(aEnvironment,aResult);
}


void doit(char*s,int doout)
{
    the_out.SetNrItems(0);
    the_out.Append('\0');
//    if (doout) printf("INPUT>%s\n",s);
    yacas->Evaluate(s);
    if (doout && the_out[0])
        printf("%s",&the_out[0]);
    if (yacas->Error()[0])
    {
        printf("ERROR>%s\n",yacas->Error());
    }
//    else

    /*
     {
        printf("RESULT>%s\n",yacas->Result());
        }
        */
}

#define IN_EXE

time_t start = 0;
int timelimit = 180;
int WallClockReached()
{
    if ((clock()-start)/CLOCKS_PER_SEC >= timelimit-10)
        return 1;
    return 0;
}

void LispWallClockNotReached(LispEnvironment& aEnvironment, LispPtr& aResult,
                   LispPtr& aArguments)
{
    if (WallClockReached())
    {
        iFinished = 1;
        InternalFalse(aEnvironment,aResult);
    }
    else
    {
        InternalTrue(aEnvironment,aResult);
    }
}

void LispWallClockTime(LispEnvironment& aEnvironment, LispPtr& aResult,
                       LispPtr& aArguments)
{
    LispChar buf[30];
    InternalIntToAscii(buf, (clock()-start)/CLOCKS_PER_SEC);
    aResult.Set(LispAtom::New(aEnvironment.HashTable().LookUp(buf)));
}
void LispTimeLimit(LispEnvironment& aEnvironment, LispPtr& aResult,
                       LispPtr& aArguments)
{
    LispChar buf[30];
    InternalIntToAscii(buf, timelimit);
    aResult.Set(LispAtom::New(aEnvironment.HashTable().LookUp(buf)));
}
void LispXXTrueType(LispEnvironment& aEnvironment, LispPtr& aResult,
                   LispPtr& aArguments)
{
    xxtokenizer.iTT = 1;
    InternalTrue(aEnvironment,aResult);
}
void LispXXNoTrueType(LispEnvironment& aEnvironment, LispPtr& aResult,
                   LispPtr& aArguments)
{
    xxtokenizer.iTT = 0;
    InternalTrue(aEnvironment,aResult);
}

int main(int argc,char** argv)
{
    if (argc>1)
    {
        timelimit = atoi(argv[1]);
//        printf("timelimit = %d\n",timelimit);
    }

    start = clock();
    yacas = CYacas::NewL(new StringOutput(the_out));
    (*yacas)()().Commands().SetAssociation(LispEvaluator(LispXXTokenizer),
                                           (*yacas)()().HashTable().LookUp("XXTokenizer"));
    (*yacas)()().Commands().SetAssociation(LispEvaluator(LispFinishPass),
                                           (*yacas)()().HashTable().LookUp("FinishPass"));

    (*yacas)()().Commands().SetAssociation(LispEvaluator(LispWallClockNotReached),
                                           (*yacas)()().HashTable().LookUp("WallClockNotReached"));


    (*yacas)()().Commands().SetAssociation(LispEvaluator(LispXXTrueType),
                                           (*yacas)()().HashTable().LookUp("XXTrueType"));
    (*yacas)()().Commands().SetAssociation(LispEvaluator(LispXXNoTrueType),
                                           (*yacas)()().HashTable().LookUp("XXNoTrueType"));


    (*yacas)()().Commands().SetAssociation(LispEvaluator(LispWallClockTime),
                                           (*yacas)()().HashTable().LookUp("XXWallClockTime"));
    (*yacas)()().Commands().SetAssociation(LispEvaluator(LispTimeLimit),
                                           (*yacas)()().HashTable().LookUp("XXTimeLimit"));




    
#include "some.inc"

#ifdef IN_EXE
#include "xxparse.inc"
#endif
    
    doit("Load(\"yacasinit.ys\");",0);
#ifndef IN_EXE
    doit("DefaultDirectory(\"./scripts/\");",0);
#endif
    doit("Load(\"xxparse.ys\");",0);

//fprintf(stderr,"start local %d\n",(clock()-start)/CLOCKS_PER_SEC);

    doit("DoRead();",1);

    while((!iFinished) && !WallClockReached())
    {
//fprintf(stderr,"ticktock %d\n",(clock()-start)/CLOCKS_PER_SEC);
        doit("DoGlobalOptimize();",1);
    }
    doit("DoEmit();",1);
    /*
     CCommandLine *commandline = new CUnixCommandLine;
    for(;;)
    {
        commandline->ReadLine("In>");
        char *inpline =  commandline->iLine.String();
        doit(inpline);
        }
        */

    delete commandline;
    delete yacas;
    return 0;
}

